// src/main.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "config.h"
#include "gettext.h"

#include "main.hh"
#include "parse_opts.hh"
#include "start_gui.hh"

#include "templates/smartp.hh"
#include "backend/history.hh"
#include "backend/schedulable.hh"
#include "backend/schedulable_list.hh"
#include "backend/schedulable_status.hh"
#include "backend/slice.hh"
#include "backend/process.hh"
#include "backend/policy.hh"
#include "backend/policy_parameters.hh"
#include "standard_io.hh"
#include "text_simulation.hh"

#include <glibmm/ustring.h>

#include <iostream>
#include <string>
#include <vector>

using namespace std;
using namespace sgpem;
using namespace memory;
using Glib::ustring;

int 
main(int argc, char* argv[]) 
{
  using namespace sgpem;

  // Set up gettext support
  setlocale(LC_ALL, "");
  bindtextdomain(PACKAGE, LOCALEDIR);
  textdomain(PACKAGE);
  
  // Set up Glib thread support
  Glib::thread_init();
  
  /*
  // Parses options and prepares vector with 
  // filenames of documents to be opened
  vector<string> filenames;
  {
    int a_count  = argc;
    char** a_ptr = argv;
    parse_options(a_count, a_ptr);
    filenames.insert(filenames.begin(), a_ptr, a_ptr+a_count);
  }
  */
    
  
  TextSimulation text_sim;
  //textual IO
  smart_ptr<IOManager> io(new StandardIO());
  text_sim.add_io_device(io);
  //grafical IO
  start_gui(argc, argv);
		  
  //SMOKE-TEST for backend classes
 /* cout << "\n\n********************************";  
  Process p1("P1", 0,10,1);
  Process p2("P2", 0,30,2);
  Process p3("P3", 5,15,3);
  Process p4("P4", 6,5,3);
  Process p5("P5", 1,10,3);
  Process p6("P6", 10,2,1);
  
  SchedulableStatus ss1(p1);	ss1.set_state(SchedulableStatus::state_running);
  SchedulableStatus ss2(p2);
  SchedulableStatus ss3(p3);
  SchedulableStatus ss4(p4);	ss4.set_state(SchedulableStatus::state_running);
  SchedulableStatus ss5(p5);	
  SchedulableStatus ss6(p6);
  
//  SimulationStatus sim1;	sim1.set_running(p1);
  
  //************** TEST HISTORY
  
  SchedulableList l1;
  l1.add_at_top(ss1);	l1.add_at_top(ss2);		l1.add_at_top(ss3);
  
  SchedulableList l2;
  l2.add_at_top(ss4);	l2.add_at_top(ss5);		l2.add_at_top(ss6);

  History h(History::get_instance());
  h.enqueue_slice(l1);	//stato iniziale
  h.enqueue_slice(l2);
  
  smart_ptr<const sgpem::SchedulableList> quale;
  
  quale = h.get_simulation_status_at(0);	//stato iniziale
 	
  cout << quale->get_item_at(0)->get_schedulable()->get_name();
  smart_ptr<const sgpem::SchedulableStatus> quale2 = h.get_scheduled_at(1);
  cout << quale2->get_schedulable()->get_name();
    
  h.truncate_at(0);
  quale = h.get_simulation_status_at(0);	//stato iniziale
 	
  cout << bool(quale) << "  " << quale->get_item_at(0)->get_schedulable()->get_name();
   */
  /*
  smart_ptr<const sgpem::SimulationStatus> quale;
  
  quale = h.get_simulation_status_at(0);
  if (quale)		cout << "\n" << quale->get_running()->get_schedulable()->get_name(); else cout << "NO";
  quale = h.get_simulation_status_at(1);
  if (quale)		cout << "\n" << quale->get_running()->get_schedulable()->get_name(); else cout << "NO";
  quale = h.get_simulation_status_at(2);
  if (quale)		cout << "\n" << quale->get_running()->get_schedulable()->get_name(); else cout << "NO";
  
  h.truncate_at(2);
  
  smart_ptr<const sgpem::SchedulableStatus> quale2;
  quale2 = h.get_scheduled_at(0);
  if (quale2)	cout << "\n" << quale2->get_schedulable()->get_name(); else cout << "NO";
  quale2 = h.get_scheduled_at(1);
  if (quale2)	cout << "\n" << quale2->get_schedulable()->get_name(); else cout << "NO";
  quale2 = h.get_scheduled_at(2);
  if (quale2)	cout << "\n" << quale2->get_schedulable()->get_name(); else cout << "NO";
  */
  
  //************** TEST QUEUE
 /* cout << "\n\nTEST QUEUE\n";
  
  SchedulableList sq;
  sq.add_at_bottom(ss1);
  sq.add_at_bottom(ss2);
  sq.add_at_bottom(ss3);
  cout << sq.get_item_at(0)->get_schedulable()->get_name() << "\n";
  cout << sq.get_item_at(1)->get_schedulable()->get_name() << "\n";
  cout << sq.get_item_at(2)->get_schedulable()->get_name() << "\n";
  sq.insert_at(2,0);
  cout << sq.get_item_at(0)->get_schedulable()->get_name() << "\n";
  cout << sq.get_item_at(1)->get_schedulable()->get_name() << "\n";
  cout << sq.get_item_at(2)->get_schedulable()->get_name() << "\n";
 
  cout << "\n\nTEST POLICYPARAMETERS\n";
  PolicyParameters pp;
  pp.register_int("ciao", 0, 100, true, 50);
  pp.set_int("ciao",1);
  cout << pp.get_int("ciao");
  
  cout << "\n\n";
  */
return 0;
}
